@@ -35,6 +35,7 @@
 #include <lasso/xml/saml-2.0/saml2_authn_statement.h>
 #include <lasso/xml/saml-2.0/saml2_audience_restriction.h>
 #include <lasso/xml/misc_text_node.h>
+#include "lasso_compat.h"
 
 /* The following are redefined in ap_config_auto.h */
 #undef PACKAGE_BUGREPORT
@@ -78,12 +79,18 @@
 #define AM_CACHE_DEFAULT_ENTRY_SIZE 196608
 #define AM_CACHE_MIN_ENTRY_SIZE 65536
 
+/* Internal error codes */
+#define AM_ERROR_INVALID_PAOS_HEADER 1
+#define AM_ERROR_MISSING_PAOS_HEADER 2
+#define AM_ERROR_MISSING_PAOS_MEDIA_TYPE 3
+
 
 /* This is the length of the id we use (for session IDs and
  * replaying POST data).
  */
 #define AM_ID_LENGTH 32
 
+#define MEDIA_TYPE_PAOS "application/vnd.paos+xml"
 
 #define am_get_srv_cfg(s) (am_srv_cfg_rec *)ap_get_module_config((s)->module_config, &auth_mellon_module)
 
@@ -91,6 +98,8 @@
 
 #define am_get_dir_cfg(r) (am_dir_cfg_rec *)ap_get_module_config((r)->per_dir_config, &auth_mellon_module)
 
+#define am_get_req_cfg(r) (am_req_cfg_rec *)ap_get_module_config((r)->request_config, &auth_mellon_module)
+
 
 typedef struct am_mod_cfg_rec {
     int cache_size;
@@ -126,12 +135,6 @@ typedef enum {
     am_enable_auth
 } am_enable_t;
 
-typedef enum {
-    am_decoder_default,
-    am_decoder_none,
-    am_decoder_feide,
-} am_decoder_t;
-
 typedef enum {
     AM_COND_FLAG_NULL = 0x000, /* No flags */
     AM_COND_FLAG_OR   = 0x001, /* Or with  next condition */
@@ -168,14 +171,11 @@ typedef struct am_dir_cfg_rec {
      */
     am_enable_t enable_mellon;
 
-    /* The decoder attribute is used to specify which decoder we should use
-     * when parsing attributes.
-     */
-    am_decoder_t decoder;
-
     const char *varname;
     int secure;
-    int merge_env_vars;
+    const char *merge_env_vars;
+    int env_vars_index_start;
+    int env_vars_count_in_n;
     const char *cookie_domain;
     const char *cookie_path;
     apr_array_header_t *cond;
@@ -241,10 +241,33 @@ typedef struct am_dir_cfg_rec {
 
     /* Cached lasso server object. */
     LassoServer *server;
+
+    /* Whether to send an ECP client a list of IdP's */
+    int ecp_send_idplist;
+
+    /* List of domains we can redirect to. */
+    const char * const *redirect_domains;
+
 } am_dir_cfg_rec;
 
+/* Bitmask for PAOS service options */
+typedef enum {
+    ECP_SERVICE_OPTION_CHANNEL_BINDING = 1,
+    ECP_SERVICE_OPTION_HOLDER_OF_KEY = 2,
+    ECP_SERVICE_OPTION_WANT_AUTHN_SIGNED = 4,
+    ECP_SERVICE_OPTION_DELEGATION = 8,
+} ECPServiceOptions;
+
+typedef struct am_req_cfg_rec {
+    char *cookie_value;
+#ifdef HAVE_ECP
+    bool ecp_authn_req;
+    ECPServiceOptions ecp_service_options;
+#endif /* HAVE_ECP */
+} am_req_cfg_rec;
+
 typedef struct am_cache_storage_t {
-    apr_uintptr_t ptr;
+    apr_size_t ptr;
 } am_cache_storage_t;
 
 typedef struct am_cache_env_t {
@@ -319,6 +342,10 @@ static const int inherit_subject_confirmation_data_address_check = -1;
 static const int default_post_replay = 0;
 static const int inherit_post_replay = -1;
 
+/* Whether to send an ECP client a list of IdP's */
+static const int default_ecp_send_idplist = 0;
+static const int inherit_ecp_send_idplist = -1;
+
 
 void *auth_mellon_dir_config(apr_pool_t *p, char *d);
 void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add);
@@ -364,6 +391,7 @@ void am_delete_request_session(request_rec *r, am_cache_entry_t *session);
 
 
 char *am_reconstruct_url(request_rec *r);
+int am_validate_redirect_url(request_rec *r, const char *url);
 int am_check_permissions(request_rec *r, am_cache_entry_t *session);
 void am_set_cache_control_headers(request_rec *r);
 int am_read_post_data(request_rec *r, char **data, apr_size_t *length);
@@ -392,7 +420,18 @@ const char *am_get_mime_header(request_rec *r, const char *m, const char *h);
 const char *am_get_mime_body(request_rec *r, const char *mime);
 char *am_get_service_url(request_rec *r, 
                          LassoProfile *profile, char *service_name);
+bool am_parse_paos_header(request_rec *r, const char *header, ECPServiceOptions *options_return);
+bool am_header_has_media_type(request_rec *r, const char *header,
+                              const char *media_type);
+const char *am_get_config_langstring(apr_hash_t *h, const char *lang);
+int am_get_boolean_query_parameter(request_rec *r, const char *name,
+                                   int *return_value, int default_value);
+char *am_get_assertion_consumer_service_by_binding(LassoProvider *provider, const char *binding);
 
+#ifdef HAVE_ECP
+char *am_ecp_service_options_str(apr_pool_t *pool, ECPServiceOptions options);
+bool am_is_paos_request(request_rec *r, int *error_code);
+#endif /* HAVE_ECP */
 
 int am_auth_mellon_user(request_rec *r);
 int am_check_uid(request_rec *r);
