@@ -597,19 +597,26 @@ void am_cache_env_populate(request_rec *r, am_cache_entry_t *t)
             apr_table_set(r->subprocess_env,prefixed_varname,value);
         }
 
-        if (d->merge_env_vars != 1) {
+        /* Check if merging of environment variables is disabled.
+         * This is either if it is NULL (default value if not configured
+         * by user) or an empty string (if specifically disabled by the user).
+         */
+        if (d->merge_env_vars == NULL || *d->merge_env_vars == '\0') {
          
             /* Add the variable with a suffix indicating how many times it has
              * been added before.
              */
             apr_table_set(r->subprocess_env,
-                          apr_psprintf(r->pool, "%s_%d", prefixed_varname, *count),
+                          apr_psprintf(r->pool, "%s_%d", prefixed_varname,
+                              (d->env_vars_index_start > -1
+                                  ? *count + d->env_vars_index_start
+                                  : *count)),
                           value);
 
         } else if (*count > 0) {
 
             /*
-             * Merge multiple values, separating with ";" 
+             * Merge multiple values, separating by default with ";"
              * this makes auth_mellon work same way mod_shib is:
              * https://wiki.shibboleth.net/confluence/display/SHIB2/NativeSPAttributeAccess
              */
@@ -617,11 +624,17 @@ void am_cache_env_populate(request_rec *r, am_cache_entry_t *t)
                            prefixed_varname,
                            apr_pstrcat(r->pool, 
                                        apr_table_get(r->subprocess_env,prefixed_varname),
-                                       ";", value, NULL));
+                                       d->merge_env_vars, value, NULL));
         }
           
         /* Increase the count. */
         ++(*count);
+
+        if (d->env_vars_count_in_n > 0) {
+             apr_table_set(r->subprocess_env,
+                           apr_pstrcat(r->pool, prefixed_varname, "_N", NULL),
+                           apr_itoa(r->pool, *count));
+        }
     }
 
     if (!am_cache_entry_slot_is_empty(&t->user)) {
