@@ -73,8 +73,20 @@ static const int post_count = 100;
 /* whether to merge env. vars or not
  * the MellonMergeEnvVars configuration directive if you change this.
  */
-static const int default_merge_env_vars = -1;
+static const char *default_merge_env_vars = NULL;
 
+/* for env. vars with multiple values, the index start
+ * the MellonEnvVarsIndexStart configuration directive if you change this.
+ */
+static const int default_env_vars_index_start = -1;
+
+/* whether to also populate env. var _N with number of values
+ * the MellonEnvVarsSetCount configuration directive if you change this.
+ */
+static const int default_env_vars_count_in_n = -1;
+
+/* The default list of trusted redirect domains. */
+static const char * const default_redirect_domains[] = { "[self]", NULL };
 
 /* This function handles configuration directives which set a 
  * multivalued string slot in the module configuration (the destination
@@ -431,8 +443,8 @@ static const char *am_set_enable_slot(cmd_parms *cmd,
 }
 
 
-/* This function handles the MellonDecoder configuration directive.
- * This directive can be set to "none" or "feide".
+/* This function handles the obsolete MellonDecoder configuration directive.
+ * It is a no-op.
  *
  * Parameters:
  *  cmd_parms *cmd       The command structure for this configuration
@@ -442,22 +454,12 @@ static const char *am_set_enable_slot(cmd_parms *cmd,
  *                       directive in the configuraion file.
  *
  * Returns:
- *  NULL on success or an error string if the argument is wrong.
+ *  NULL
  */
 static const char *am_set_decoder_slot(cmd_parms *cmd,
                                        void *struct_ptr,
                                        const char *arg)
 {
-    am_dir_cfg_rec *d = (am_dir_cfg_rec *)struct_ptr;
-
-    if(!strcasecmp(arg, "none")) {
-        d->decoder = am_decoder_none;
-    } else if(!strcasecmp(arg, "feide")) {
-        d->decoder = am_decoder_feide;
-    } else {
-        return "MellonDecoder must be 'none' or 'feide'";
-    }
-
     return NULL;
 }
 
@@ -850,6 +852,89 @@ static const char *am_set_do_not_verify_logout_signature(cmd_parms *cmd,
 #endif
 }
 
+/* This function handles the MellonMergeEnvVars configuration directive,
+ * it sets merge_env_vars to nonempty separator (default semicolon),
+ * or empty string to denote no merging.
+ *
+ * Parameters:
+ *  cmd_parms *cmd       The command structure for this configuration
+ *                       directive.
+ *  void *struct_ptr     Pointer to the current directory configuration.
+ *                       NULL if we are not in a directory configuration.
+ *  const char *flag     On/Off flag
+ *  const char *sep      Optional separator, should be only present with On
+ *
+ * Returns:
+ *  NULL on success or an error string on failure.
+ */
+static const char *am_set_merge_env_vars(cmd_parms *cmd,
+                                          void *struct_ptr,
+                                          const char *flag,
+                                          const char *sep)
+{
+    am_dir_cfg_rec *d = (am_dir_cfg_rec *)struct_ptr;
+    apr_pool_t *p= cmd->pool;
+    if (strcasecmp(flag, "on") == 0) {
+        if (sep && *sep) {
+            /*
+             * TAKE12 will not give us the second argument if it is
+             * empty string so we cannot complain about it, we will just
+             * silently use semicolon
+             */
+            d->merge_env_vars = apr_pstrdup(p, sep);
+        } else {
+            d->merge_env_vars = ";";
+        }
+    } else if (strcasecmp(flag, "off") == 0) {
+        if (sep) {
+            return apr_pstrcat(cmd->pool, cmd->cmd->name,
+                " separator should not be used with Off", NULL);
+        }
+        d->merge_env_vars = "";
+    } else {
+        return apr_pstrcat(cmd->pool, cmd->cmd->name,
+            " first parameer must be On or Off", NULL);
+    }
+    return NULL;
+}
+
+/* Handle MellonRedirectDomains option.
+ *
+ * Parameters:
+ *  cmd_parms *cmd       The command structure for this configuration
+ *                       directive.
+ *  void *struct_ptr     Pointer to the current directory configuration.
+ *                       NULL if we are not in a directory configuration.
+ *  int argc             Number of redirect domains.
+ *  char *const argv[]   List of redirect domains.
+ *
+ * Returns:
+ *  NULL on success, or errror string on failure.
+ */
+static const char *am_set_redirect_domains(cmd_parms *cmd,
+                                          void *struct_ptr,
+                                          int argc,
+                                          char *const argv[])
+{
+    am_dir_cfg_rec *cfg = (am_dir_cfg_rec *)struct_ptr;
+    const char **redirect_domains;
+    int i;
+
+    if (argc < 1)
+        return apr_psprintf(cmd->pool, "%s takes at least one arguments",
+                            cmd->cmd->name);
+
+    redirect_domains = apr_palloc(cmd->pool, sizeof(const char *) * (argc + 1));
+    for (i = 0; i < argc; i++) {
+        redirect_domains[i] = argv[i];
+    }
+    redirect_domains[argc] = NULL;
+
+    cfg->redirect_domains = redirect_domains;
+
+    return NULL;
+}
+
 /* This array contains all the configuration directive which are handled
  * by auth_mellon.
  */    
@@ -935,11 +1020,7 @@ const command_rec auth_mellon_commands[] = {
         am_set_decoder_slot,
         NULL,
         OR_AUTHCFG,
-        "Select which decoder mod_auth_mellon should use to decode attribute"
-        " values. This option can be se to either 'none' or 'feide'. 'none'"
-        " is the default, and will store the attributes as they are received"
-        " from the IdP. 'feide' is for decoding base64-encoded values which"
-        " are separated by a underscore."
+        "Obsolete option, now a no-op for backwards compatibility."
         ),
     AP_INIT_TAKE1(
         "MellonVariable",
@@ -1224,12 +1305,42 @@ const command_rec auth_mellon_commands[] = {
         OR_AUTHCFG,
         "Whether we should replay POST requests that trigger authentication. Default is off."
         ),
-    AP_INIT_FLAG(
+    AP_INIT_TAKE12(
         "MellonMergeEnvVars",
-        ap_set_flag_slot,
-        (void *)APR_OFFSETOF(am_dir_cfg_rec, merge_env_vars),
+        am_set_merge_env_vars,
+        NULL,
         OR_AUTHCFG,
-        "Whether to merge environement variables multi-values or not. Default is off."
+        "Whether to merge environment variables multi-values or not. Default is off."
+        "When first parameter is on, optional second parameter is the separator, "
+        "defaulting to semicolon."
+        ),
+    AP_INIT_TAKE1(
+        "MellonEnvVarsIndexStart",
+        ap_set_int_slot,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, env_vars_index_start),
+        OR_AUTHCFG,
+        "Start indexing environment variables for multivalues with 0 or 1. Default is 0."
+        ),
+    AP_INIT_FLAG(
+        "MellonEnvVarsSetCount",
+        ap_set_flag_slot,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, env_vars_count_in_n),
+        OR_AUTHCFG,
+        "Whether to also populate environment variable suffixed _N with number of values. Default is off."
+        ),
+    AP_INIT_FLAG(
+        "MellonECPSendIDPList",
+        ap_set_flag_slot,
+        (void *)APR_OFFSETOF(am_dir_cfg_rec, ecp_send_idplist),
+        OR_AUTHCFG,
+        "Whether to send an ECP client a list of IdP's. Default is off."
+        ),
+    AP_INIT_TAKE_ARGV(
+        "MellonRedirectDomains",
+        am_set_redirect_domains,
+        NULL,
+        OR_AUTHCFG,
+        "List of domains we can redirect to."
         ),
     {NULL}
 };
@@ -1282,11 +1393,11 @@ void *auth_mellon_dir_config(apr_pool_t *p, char *d)
 
     dir->enable_mellon = am_enable_default;
 
-    dir->decoder = am_decoder_default;
-
     dir->varname = default_cookie_name;
     dir->secure = default_secure_cookie;
     dir->merge_env_vars = default_merge_env_vars;
+    dir->env_vars_index_start = default_env_vars_index_start;
+    dir->env_vars_count_in_n = default_env_vars_count_in_n;
     dir->cond = apr_array_make(p, 0, sizeof(am_cond_t));
     dir->cookie_domain = NULL;
     dir->cookie_path = NULL;
@@ -1327,6 +1438,9 @@ void *auth_mellon_dir_config(apr_pool_t *p, char *d)
     dir->subject_confirmation_data_address_check = inherit_subject_confirmation_data_address_check;
     dir->do_not_verify_logout_signature = apr_hash_make(p);
     dir->post_replay = inherit_post_replay;
+    dir->redirect_domains = default_redirect_domains;
+
+    dir->ecp_send_idplist = inherit_ecp_send_idplist;
 
     return dir;
 }
@@ -1393,11 +1507,6 @@ void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add)
                               base_cfg->enable_mellon);
 
 
-    new_cfg->decoder = (add_cfg->decoder != am_decoder_default ?
-                        add_cfg->decoder :
-                        base_cfg->decoder);
-
-
     new_cfg->varname = (add_cfg->varname != default_cookie_name ?
                         add_cfg->varname :
                         base_cfg->varname);
@@ -1411,6 +1520,14 @@ void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add)
                                add_cfg->merge_env_vars :
                                base_cfg->merge_env_vars);
 
+    new_cfg->env_vars_index_start = (add_cfg->env_vars_index_start != default_env_vars_index_start ?
+                               add_cfg->env_vars_index_start :
+                               base_cfg->env_vars_index_start);
+
+    new_cfg->env_vars_count_in_n = (add_cfg->env_vars_count_in_n != default_env_vars_count_in_n ?
+                               add_cfg->env_vars_count_in_n :
+                               base_cfg->env_vars_count_in_n);
+
     new_cfg->cookie_domain = (add_cfg->cookie_domain != NULL ?
                         add_cfg->cookie_domain :
                         base_cfg->cookie_domain);
@@ -1554,6 +1671,13 @@ void *auth_mellon_dir_merge(apr_pool_t *p, void *base, void *add)
         CFG_MERGE(add_cfg, base_cfg, subject_confirmation_data_address_check);
     new_cfg->post_replay = CFG_MERGE(add_cfg, base_cfg, post_replay);
 
+    new_cfg->ecp_send_idplist = CFG_MERGE(add_cfg, base_cfg, ecp_send_idplist);
+
+    new_cfg->redirect_domains =
+        (add_cfg->redirect_domains != default_redirect_domains ?
+         add_cfg->redirect_domains :
+         base_cfg->redirect_domains);
+
     return new_cfg;
 }
 
